@@ -51,7 +51,6 @@
 #include "types.h"
 #include "utf8.h"
 #include "buffer.h"
-#include "mode.h"
 #include "locus.h"
 #include "text.h"
 #include "window.h"
@@ -59,20 +58,7 @@
 #include "clip.h"
 #include "macro.h"
 #include "display.h"
-
-/* Miscellaneous declarations and prototypes that didn't fit elsewhere */
-extern struct termios original_termios;
-
-void *reallocate(const void *, size_t);		/* mem.c */
-#define allocate(sz) (reallocate(NULL, (sz)))
-void *allocate0(size_t);
-#define RELEASE(p) (reallocate((p), 0), (p) = NULL)
-
-void depart(int exit_status);			/* die.c */
-void die(const char *, ...);
-void message(const char *, ...);
-void status(const char *, ...);
-void status_hide(void);
-
-Boolean_t multiplexor(Boolean_t block);		/* child.c */
-void multiplex_write(fd_t fd, const char *, ssize_t bytes, Boolean_t retain);
+#include "mode.h"
+#include "mem.h"
+#include "die.h"
+#include "child.h"
