@@ -29,10 +29,8 @@
 #define CTL_RESETCOLORS	CSI "39;49m"
 #define CTL_GOTO	CSI "%d;%df"
 #define CTL_ERASEALL	CSI "2J"
-#if 0 /* broken on Apple Terminal, dammit */
-# define CTL_ERASETOEND	CSI "J"
-# define CTL_ERASELINE	CSI "K"
-#endif
+#define CTL_ERASETOEND	CSI "J"
+#define CTL_ERASELINE	CSI "K"
 #define CTL_ERASECOLS	CSI "%dX"
 #define CTL_DELCOLS	CSI "%uP"
 #define CTL_DELLINES	CSI "%uM"
@@ -49,6 +47,8 @@
 #define XTERM_BCKISDEL	CSI "?67l"
 #define XTERM_LOCATOR	CSI "1'z" CSI "1'{" CSI "4'{"
 
+#define BAD_RGBA 0x01010100
+
 
 struct cell {
 	Unicode_t unicode;
@@ -66,20 +66,28 @@ struct display {
 	Byte_t inbuf[INBUF_SIZE];
 	char outbuf[OUTBUF_SIZE];
 	size_t inbuf_bytes, outbuf_bytes;
-	Boolean_t is_xterm;
+	Boolean_t is_xterm, is_linux;
 	rgba_t color[MAX_COLORS];
 	unsigned colors;
 	char *title;
 };
 
+struct termios original_termios;
 static struct display *display_list;
 static void (*old_sigwinch)(int, siginfo_t *, void *);
+static FILE *debug_file;
 
 static void emit(const char *str, size_t bytes)
 {
 	size_t wrote;
 	ssize_t chunk;
 
+	if (debug_file && bytes) {
+		fprintf(debug_file, "emit %d:", (int) bytes);
+		fwrite(str, bytes, 1, debug_file);
+		fputc('\n', debug_file);
+	}
+
 	for (wrote = 0; wrote < bytes; wrote += chunk) {
 		errno = 0;
 		chunk = write(1, str + wrote, bytes - wrote);
@@ -124,6 +132,12 @@ static void outf(struct display *display, const char *msg, ...)
 	outs(display, buf);
 }
 
+static void force_moveto(struct display *display, unsigned row, unsigned column)
+{
+	outf(display, CTL_GOTO, (display->at_row = row) + 1,
+	     (display->at_column = column) + 1);
+}
+
 static void moveto(struct display *display, unsigned row, unsigned column)
 {
 	if (row == display->at_row && column == display->at_column)
@@ -141,8 +155,7 @@ static void moveto(struct display *display, unsigned row, unsigned column)
 			return;
 		}
 	}
-	outf(display, CTL_GOTO, (display->at_row = row) + 1,
-	     (display->at_column = column) + 1);
+	force_moveto(display, row, column);
 }
 
 void display_sync(struct display *display)
@@ -232,7 +245,9 @@ static unsigned colormap(struct display *display, rgba_t rgba)
 
 static void set_color(struct display *display, rgba_t rgba, unsigned magic)
 {
-	if (display->is_xterm) {
+	if (display->is_linux)
+		outf(display, CSI "%dm", linux_colormap(rgba) + magic);
+	else {
 		unsigned c = colormap(display, rgba);
 		if (c >= 18)
 			outf(display, CSI "%d;5;%dm", magic+8, c);
@@ -240,8 +255,7 @@ static void set_color(struct display *display, rgba_t rgba, unsigned magic)
 			outf(display, CSI "%dm", c - 10 + magic + 60);
 		else
 			outf(display, CSI "%dm", c + magic);
-	} else
-		outf(display, CSI "%dm", linux_colormap(rgba) + magic);
+	}
 }
 
 static void background_color(struct display *display, rgba_t rgba)
@@ -270,39 +284,45 @@ void display_put(struct display *display, unsigned row, unsigned column,
 	if (unicode < ' ')
 		unicode = ' ';
 	cell = &display->image[row*display->columns + column];
-	if (cell->unicode != unicode ||
-	    cell->fgrgba != fgrgba && unicode != ' ' ||
-	    cell->bgrgba != bgrgba) {
-		moveto(display, row, column);
-		background_color(display, bgrgba);
-		if (unicode != ' ')
-			foreground_color(display, fgrgba);
-		out(display, buf, unicode_utf8(buf, unicode));
-		display->at_column++;
-		cell->unicode = unicode;
-		cell->bgrgba = bgrgba;
-	}
+	if (cell->unicode == unicode &&
+	    (cell->fgrgba == fgrgba || unicode == ' ') &&
+	    cell->bgrgba == bgrgba)
+		return;
+	moveto(display, row, column);
+	background_color(display, bgrgba);
+	if (unicode != ' ')
+		foreground_color(display, fgrgba);
+	out(display, buf, unicode_utf8(buf, unicode));
+	display->at_column++;
+	/* display->image may have moved due to resize */
+	cell = &display->image[row*display->columns + column];
+	cell->unicode = unicode;
+	cell->bgrgba = bgrgba;
 	cell->fgrgba = fgrgba;
 }
 
-static void fill(struct display *display, unsigned row, unsigned column,
-		 unsigned columns, Unicode_t code, rgba_t fgrgba,
-		 rgba_t bgrgba)
+static void clear_image(struct display *display, unsigned row, unsigned rows,
+			unsigned column, unsigned columns,
+			Boolean_t colors_trashed)
 {
-	struct cell *cell = &display->image[row * display->columns +
-					    column];
-	while (columns--) {
-		cell->unicode = code;
-		cell->fgrgba = fgrgba;
-		cell++->bgrgba = bgrgba;
+	int j;
+	while (rows--) {
+		struct cell *cell = &display->image[row++ * display->columns +
+						    column];
+		for (j = 0; j < columns; j++, cell++) {
+			cell->unicode = ' ';
+			if (colors_trashed) {
+				cell->fgrgba = BAD_RGBA;
+				cell->bgrgba = BAD_RGBA;
+			}
+		}
 	}
 }
 
 void display_erase(struct display *display, unsigned row, unsigned column,
-		   unsigned rows, unsigned columns,
-		   rgba_t fgrgba, rgba_t bgrgba)
+		   unsigned rows, unsigned columns)
 {
-	int r, c;
+	int r;
 
 	if (row >= display->rows || column >= display->columns)
 		return;
@@ -313,46 +333,32 @@ void display_erase(struct display *display, unsigned row, unsigned column,
 	if (!columns)
 		return;
 
-	for (r = 0; r < rows; r++) {
-		struct cell *cell = &display->image[(row+r)*display->columns +
-						    column];
-		for (c = 0; c < columns; c++)
-			if (cell->unicode != ' ' ||
-			    cell->fgrgba != fgrgba ||
-			    cell++->bgrgba != bgrgba)
-				goto doit;
-	}
-	return;
-
-doit:	foreground_color(display, fgrgba);
-	background_color(display, bgrgba);
-
-#ifdef CTL_ERASETOEND
-	if (!column && columns == display->columns && row + rows == display->rows) {
-		moveto(display, row, column);
-		outs(display, CTL_ERASETOEND);
-	} else
-#endif
-#ifdef CTL_ERASELINE
-	if (column + columns == display->columns) {
+	if (!column &&
+	    columns == display->columns &&
+	    row + rows == display->rows) {
+		if (row) {
+			moveto(display, row, column);
+			outs(display, CTL_ERASETOEND);
+		} else
+			outs(display, CTL_ERASEALL);
+		clear_image(display, row, rows, column, columns, FALSE);
+	} else if (column + columns == display->columns) {
 		for (r = 0; r < rows; r++) {
 			moveto(display, row + r, column);
 			outs(display, CTL_ERASELINE);
 		}
-	} else
-#endif
-	for (r = 0; r < rows; r++) {
-		moveto(display, row + r, column);
-		outf(display, CTL_ERASECOLS, columns);
+		clear_image(display, row, rows, column, columns, TRUE);
+	} else {
+		for (r = 0; r < rows; r++) {
+			moveto(display, row + r, column);
+			outf(display, CTL_ERASECOLS, columns);
+		}
+		clear_image(display, row, rows, column, columns, FALSE);
 	}
-
-	for (; rows--; row++)
-		fill(display, row, column, columns, ' ', fgrgba, bgrgba);
 }
 
 void display_insert_spaces(struct display *display, unsigned row,
-			   unsigned column, unsigned spaces, unsigned columns,
-			   rgba_t fgrgba, rgba_t bgrgba)
+			   unsigned column, unsigned spaces, unsigned columns)
 {
 	struct cell *cell;
 
@@ -364,22 +370,19 @@ void display_insert_spaces(struct display *display, unsigned row,
 		spaces = columns;
 	if (!spaces)
 		return;
-
 	if (column + columns != display->columns) {
 		moveto(display, row, column + columns - spaces);
 		outf(display, CTL_DELCOLS, spaces);
 	}
 	moveto(display, row, column);
-	background_color(display, bgrgba);
 	outf(display, CTL_INSCOLS, spaces);
 	cell = &display->image[row*display->columns + column];
 	memmove(cell + spaces, cell, (columns - spaces) * sizeof *cell);
-	fill(display, row, column, spaces, ' ', fgrgba, bgrgba);
+	clear_image(display, row, 1, column, spaces, FALSE);
 }
 
 void display_delete_chars(struct display *display, unsigned row,
-			  unsigned column, unsigned chars, unsigned columns,
-			  rgba_t fgrgba, rgba_t bgrgba)
+			  unsigned column, unsigned chars, unsigned columns)
 {
 	struct cell *cell;
 
@@ -393,14 +396,12 @@ void display_delete_chars(struct display *display, unsigned row,
 		return;
 
 	moveto(display, row, column);
-	background_color(display, bgrgba);
 	outf(display, CTL_DELCOLS, chars);
 	moveto(display, row, column + columns - chars);
 	outf(display, CTL_INSCOLS, chars);
 	cell = &display->image[row*display->columns + column];
 	memmove(cell, cell + chars, (columns - chars) * sizeof *cell);
-	fill(display, row, column + columns - chars, chars,
-	     ' ', fgrgba, bgrgba);
+	clear_image(display, row, 1, column + columns - chars, chars, FALSE);
 }
 
 static Boolean_t validate(struct display *display, unsigned row,
@@ -419,41 +420,35 @@ static Boolean_t validate(struct display *display, unsigned row,
 }
 
 static void insert_whole_lines(struct display *display, unsigned row,
-			       unsigned lines, unsigned rows,
-			       rgba_t fgrgba, rgba_t bgrgba)
+			       unsigned lines, unsigned rows)
 {
-	struct cell *cell = &display->image[row * display->columns];
-
 	if (row + rows != display->rows) {
 		moveto(display, row + rows - lines, 0);
 		outf(display, CTL_DELLINES, lines);
 	}
 	moveto(display, row, 0);
-	background_color(display, bgrgba);
 	outf(display, CTL_INSLINES, lines);
 	memmove(&display->image[(row + lines) * display->columns],
-		cell, (rows - lines) * display->columns * sizeof *cell);
-	fill(display, row, 0, lines * display->columns, ' ', fgrgba, bgrgba);
+		&display->image[row * display->columns],
+		(rows - lines) * display->columns * sizeof *display->image);
+	clear_image(display, row, lines, 0, display->columns, FALSE);
 }
 
 static void delete_whole_lines(struct display *display, unsigned row,
-			       unsigned lines, unsigned rows,
-			       rgba_t fgrgba, rgba_t bgrgba)
+			       unsigned lines, unsigned rows)
 {
-	struct cell *cell = &display->image[row * display->columns];
+	struct cell *cell;  /* assigned later for safety from resizing */
 
 	moveto(display, row, 0);
-	background_color(display, bgrgba);
 	outf(display, CTL_DELLINES, lines);
 	if (row + rows != display->rows) {
 		moveto(display, row + rows - lines, 0);
-		background_color(display, bgrgba);
 		outf(display, CTL_INSLINES, lines);
 	}
+	cell = &display->image[row * display->columns];
 	memmove(cell, cell + lines * display->columns,
 		(rows - lines) * display->columns * sizeof *cell);
-	fill(display, row + rows - lines, 0, lines * display->columns,
-	     ' ', fgrgba, bgrgba);
+	clear_image(display, row + rows - lines, lines, 0, display->columns, FALSE);
 }
 
 static void shortpause(struct display *display, int millisec)
@@ -467,15 +462,13 @@ static void shortpause(struct display *display, int millisec)
 
 static void whole_lines(struct display *display, unsigned row,
 			unsigned lines, unsigned rows,
-			rgba_t fgrgba, rgba_t bgrgba,
 			void (*mover)(struct display *, unsigned, unsigned,
-				      unsigned, rgba_t, rgba_t))
+				      unsigned))
 {
 	unsigned amount;
 	for (; lines; lines -= amount) {
 		amount = 1 + (lines > 2) + (lines > 4) + (lines > 8);
-		mover(display, row, amount = 1 + (lines > 2),
-		      rows, fgrgba, bgrgba);
+		mover(display, row, amount = 1 + (lines > 2), rows);
 		shortpause(display, 12 - 4 * amount);
 	}
 }
@@ -483,52 +476,57 @@ static void whole_lines(struct display *display, unsigned row,
 static void copy_line(struct display *display, unsigned to_row,
 		      unsigned from_row, unsigned column, unsigned columns)
 {
-	struct cell *cell = &display->image[from_row * display->columns +
-					    column];
-	for (; columns--; cell++)
+	if (to_row >= display->rows ||
+	    from_row >= display->rows ||
+	    to_row == from_row ||
+	    column >= display->columns)
+		return;
+	if (columns > display->columns - column)
+		columns = display->columns - column;
+	while (columns--) {
+		struct cell *cell =
+			&display->image[from_row * display->columns +
+					column];
 		display_put(display, to_row, column++,
 			    cell->unicode, cell->fgrgba, cell->bgrgba);
+	}
 }
 
 void display_insert_lines(struct display *display, unsigned row,
 			  unsigned column, unsigned lines,
-			  unsigned rows, unsigned columns,
-			  rgba_t fgrgba, rgba_t bgrgba)
+			  unsigned rows, unsigned columns)
 {
 	int j;
-
 	if (!validate(display, row, column, &rows, &columns, &lines))
 		return;
 	if (!column && columns == display->columns)
-		whole_lines(display, row, lines, rows, fgrgba, bgrgba,
+		whole_lines(display, row, lines, rows,
 			    insert_whole_lines);
 	else {
 		for (j = rows - lines; j-- > 0; )
 			copy_line(display, row + lines + j, row + j,
 				  column, columns);
-		display_erase(display, row, column, lines, columns,
-			      fgrgba, bgrgba);
+		display_erase(display, row, column, lines, columns);
 	}
 }
 
 void display_delete_lines(struct display *display, unsigned row,
 			  unsigned column, unsigned lines,
-			  unsigned rows, unsigned columns,
-			  rgba_t fgrgba, rgba_t bgrgba)
+			  unsigned rows, unsigned columns)
 {
 	int j;
 
 	if (!validate(display, row, column, &rows, &columns, &lines))
 		return;
 	if (!column && columns == display->columns)
-		whole_lines(display, row, lines, rows, fgrgba, bgrgba,
+		whole_lines(display, row, lines, rows,
 			    delete_whole_lines);
 	else {
 		for (j = 0; j < rows - lines; j++)
 			copy_line(display, row + j, row + lines + j,
 				  column, columns);
 		display_erase(display, row + rows - lines, column,
-			      lines, columns, fgrgba, bgrgba);
+			      lines, columns);
 	}
 }
 
@@ -537,7 +535,7 @@ static struct cell *resize(struct display *display, struct cell *old,
 			   unsigned new_rows, unsigned new_columns)
 {
 	unsigned cells = new_rows * new_columns;
-	size_t bytes = cells * sizeof(struct cell);
+	size_t bytes = cells * sizeof *old;
 	struct cell *new = allocate(bytes);
 	unsigned min_rows = new_rows < old_rows ? new_rows : old_rows;
 	unsigned min_columns = new_columns < old_columns ?
@@ -589,6 +587,7 @@ static void set_geometry(struct display *display,
 	display->rows = rows;
 	display->columns = columns;
 	display->size_changed = TRUE;
+	display_erase(display, 0, rows, 0, columns);
 	display_cursor(display, display->cursor_row, display->cursor_column);
 }
 
@@ -620,7 +619,7 @@ static void geometry(struct display *display)
 			rows = 24;
 		if (!columns)
 			columns = 80;
-		moveto(display, 666, 666);
+		force_moveto(display, 666, 666);
 		outs(display, CTL_CURSORPOS);
 	}
 	set_geometry(display, rows, columns);
@@ -637,25 +636,24 @@ void display_get_geometry(struct display *display,
 void display_reset(struct display *display)
 {
 	RELEASE(display->image);
-	display->cursor_row = display->cursor_column = 0;
-	display->at_row = display->at_column = 0;
 	if (display->is_xterm)
 		outs(display, XTERM_ALTSCREEN);
 	else
 		outs(display, CTL_RESET);
 	outs(display, CTL_UTF8
 		      CTL_RESETMODES
-		      CTL_RESETCOLORS
-		      CTL_ERASEALL);
+		      CTL_RESETCOLORS);
 	if (display->is_xterm)
 		outs(display, XTERM_BCKISDEL);
 	else
 		outs(display, CTL_NUMLOCK CTL_CLEARLEDS CTL_NUMLOCKLED);
 	display->colors = 0;
-	display->fgrgba = DEFAULT_FGRGBA;
-	display->bgrgba = DEFAULT_BGRGBA;
+	display->fgrgba = BAD_RGBA;
+	display->bgrgba = BAD_RGBA;
 	geometry(display);
-	display_sync(display);
+	force_moveto(display, 0, 0);
+	display->cursor_row = display->cursor_column = 0;
+	flush(display);
 }
 
 static void sigwinch(int signo, siginfo_t *info, void *data)
@@ -682,7 +680,11 @@ struct display *display_init(void)
 {
 	struct display *display = allocate0(sizeof *display);
 	struct termios termios = original_termios;
-	const char *term;
+	const char *term, *path;
+
+	if ((path = getenv("DISPLAY_DEBUG_PATH")) &&
+	    !(debug_file = fopen(path, "w")))
+		die("could not open display debug file %s for writing", path);
 
 	cfmakeraw(&termios);
 	tcsetattr(1, TCSADRAIN, &termios);
@@ -701,10 +703,10 @@ struct display *display_init(void)
 
 	display_list = display;
 	display->inbuf_bytes = display->outbuf_bytes = 0;
-	display->is_xterm = (term = getenv("TERM")) &&
-			    !strncmp(term, "xterm", 5);
+	term = getenv("TERM");
+	if (!(display->is_xterm = term && !strncmp(term, "xterm", 5)))
+		display->is_linux = term && !strcmp(term, "linux");
 	display_reset(display);
-	display_sync(display);
 	return display;
 }
 
@@ -716,8 +718,8 @@ void display_end(struct display *display)
 		return;
 
 	display_title(display, NULL);
-	set_color(display, DEFAULT_FGRGBA, BG_COLOR);
-	set_color(display, DEFAULT_BGRGBA, FG_COLOR);
+	set_color(display, DEFAULT_FGRGBA, FG_COLOR);
+	set_color(display, DEFAULT_BGRGBA, BG_COLOR);
 	if (display->is_xterm)
 		outs(display, XTERM_REGSCREEN);
 	else
@@ -765,8 +767,8 @@ void display_cursor(struct display *display, unsigned row, unsigned column)
 		row = display->rows - 1;
 	if (column >= display->columns)
 		column = display->columns - 1;
-	moveto(display, display->cursor_row = row,
-		display->cursor_column = column);
+	display->cursor_row = row;
+	display->cursor_column = column;
 }
 
 void display_beep(struct display *display)
@@ -802,6 +804,13 @@ again:	if (display->size_changed)
 				 sizeof display->inbuf - 1 -
 					display->inbuf_bytes);
 		} while (n < 0 && (errno == EAGAIN || errno == EINTR));
+		if (debug_file) {
+			fprintf(debug_file, "read %d:", n);
+			if (n > 0)
+				fwrite(display->inbuf + display->inbuf_bytes,
+				       n, 1, debug_file);
+			fputc('\n', debug_file);
+		}
 		if (!n)
 			return ERROR_EOF;
 		if (n < 0)
@@ -846,7 +855,7 @@ again:	if (display->size_changed)
 				key = SET_GEOMETRY;
 			break;
 		case '~':
-			if (!val)
+			if (!val[0])
 				break;
 			switch (val[0]) {
 			case  2: key = FUNCTION_INSERT;	break;
