@@ -2,6 +2,8 @@
 #ifndef DISPLAY_H
 #define DISPLAY_H
 
+extern struct termios original_termios;
+
 struct display;
 
 typedef unsigned rgba_t;
@@ -16,9 +18,6 @@ void display_title(struct display *, const char *);
 void display_cursor(struct display *, unsigned row, unsigned column);
 void display_put(struct display *, unsigned row, unsigned column,
 		 Unicode_t unicode, rgba_t fgRGBA, rgba_t bgRGBA);
-void display_erase(struct display *, unsigned row, unsigned column,
-		   unsigned rows, unsigned columns,
-		   rgba_t fgRGBA, rgba_t bgRGBA);
 void display_beep(struct display *);
 void display_sync(struct display *);
 
@@ -30,17 +29,15 @@ void display_sync(struct display *);
 Unicode_t display_getch(struct display *, Boolean_t block);
 
 /* hints */
+void display_erase(struct display *, unsigned row, unsigned column,
+		   unsigned rows, unsigned columns);
 void display_insert_spaces(struct display *, unsigned row, unsigned column,
-			   unsigned spaces, unsigned columns,
-			   rgba_t fgRGBA, rgba_t bgRGBA);
+			   unsigned spaces, unsigned columns);
 void display_delete_chars(struct display *, unsigned row, unsigned column,
-			  unsigned chars, unsigned columns,
-			  rgba_t fgRGBA, rgba_t bgRGBA);
+			  unsigned chars, unsigned columns);
 void display_insert_lines(struct display *, unsigned row, unsigned column,
-			  unsigned lines, unsigned rows, unsigned columns,
-			  rgba_t fgRGBA, rgba_t bgRGBA);
+			  unsigned lines, unsigned rows, unsigned columns);
 void display_delete_lines(struct display *, unsigned row, unsigned column,
-			  unsigned lines, unsigned rows, unsigned columns,
-			  rgba_t fgRGBA, rgba_t bgRGBA);
+			  unsigned lines, unsigned rows, unsigned columns);
 
 #endif
