@@ -17,6 +17,7 @@ struct window {
 	unsigned last_dirties;
 	position_t last_cursor, last_mark;
 	rgba_t last_fgrgba, last_bgrgba;
+	struct mode *last_mode;
 	struct window *next;
 };
 
@@ -170,8 +171,7 @@ struct window *window_raise(struct view *view)
 	if (!display)
 		display = display_init();
 	display_get_geometry(display, &display_rows, &display_columns);
-	display_erase(display, 0, 0, display_rows, display_columns,
-		      DEFAULT_FGRGBA, DEFAULT_BGRGBA);
+	display_erase(display, 0, 0, display_rows, display_columns);
 	while (window_list && window_list->view != view)
 		window_destroy(window_list);
 	while (window_list && window_list->next)
@@ -412,7 +412,7 @@ static unsigned paintch(struct window *window, Unicode_t ch, unsigned row,
 	if (mark != UNSET &&
 	    (at >= cursor && at < mark ||
 	     at >= mark && at < cursor)) {
-		bgrgba = 0x00ffff00;
+		bgrgba = window->view->mode->selection_bgrgba;
 		fgrgba = 0xff000000;
 	} else if (at == cursor) {
 		if (window->view->text->flags & TEXT_RDONLY)
@@ -446,8 +446,8 @@ static unsigned paintch(struct window *window, Unicode_t ch, unsigned row,
 			ch += '@';
 	} else if (ch == ' ') {
 		if (bgrgba == window->bgrgba &&
-		   lame_space(window->view, at + 1,
-			      tabstop-1 - column % tabstop))
+		    lame_space(window->view, at + 1,
+			       tabstop-1 - column % tabstop))
 			bgrgba = 0xff00ff00;
 	} else if (!IS_UNICODE(ch))
 		ch = ' ', bgrgba = 0xff00ff00;
@@ -476,7 +476,8 @@ static Boolean_t needs_repainting(struct window *window)
 		window->last_fgrgba != window->fgrgba ||
 		window->last_bgrgba != window->bgrgba ||
 		window->last_cursor != cursor ||
-		window->last_mark != mark;
+		window->last_mark != mark ||
+		window->last_mode != view->mode;
 }
 
 static void repainted(struct window *window, position_t cursor, position_t mark)
@@ -486,6 +487,7 @@ static void repainted(struct window *window, position_t cursor, position_t mark)
 	window->last_bgrgba = window->bgrgba;
 	window->last_cursor = cursor;
 	window->last_mark = mark;
+	window->last_mode = window->view->mode;
 }
 
 static void paint(struct window *window)
@@ -524,10 +526,15 @@ static void paint(struct window *window)
 					 fgrgba);
 		}
 
+#if 0
 		display_erase(display, window->row + row,
 			      window->column + column, 1,
-			      window->columns - column,
-			      window->fgrgba, window->bgrgba);
+			      window->columns - column);
+#endif
+		while (column < window->columns)
+			display_put(display, window->row + row,
+				    window->column + column++,
+				    ' ', window->fgrgba, window->bgrgba);
 	}
 
 	repainted(window, cursor, mark);
@@ -562,8 +569,7 @@ void window_hint_deleting(struct window *window, position_t offset, size_t bytes
 			display_delete_chars(display, window->row + row,
 					     window->column + column,
 					     end_column - column,
-					     window->columns, window->fgrgba,
-					     window->bgrgba);
+					     window->columns - column);
 			return;
 		}
 		lines++;
@@ -573,8 +579,7 @@ void window_hint_deleting(struct window *window, position_t offset, size_t bytes
 		end_column = 0;
 	}
 	display_delete_lines(display, window->row + row, window->column,
-			     lines, window->rows, window->columns,
-			     window->fgrgba, window->bgrgba);
+			     lines, window->rows, window->columns);
 }
 
 void window_hint_inserted(struct window *window, position_t offset, size_t bytes)
@@ -606,8 +611,7 @@ void window_hint_inserted(struct window *window, position_t offset, size_t bytes
 			display_insert_spaces(display, window->row + row,
 					      window->column + column,
 					      end_column - column,
-					      window->columns, window->fgrgba,
-					      window->bgrgba);
+					      window->columns - column);
 			return;
 		}
 		lines++;
@@ -617,8 +621,7 @@ void window_hint_inserted(struct window *window, position_t offset, size_t bytes
 		end_column = 0;
 	}
 	display_insert_lines(display, window->row + row, window->column,
-			     lines, window->rows, window->columns,
-			     window->fgrgba, window->bgrgba);
+			     lines, window->rows, window->columns);
 }
 
 void window_next(struct view *view)
@@ -627,6 +630,14 @@ void window_next(struct view *view)
 		activate(view->window->next ? view->window->next : window_list);
 }
 
+void window_index(int num)
+{
+	struct window *window = window_list;
+	while (--num > 0 && window->next)
+		window = window->next;
+	activate(window);
+}
+
 struct window *window_recenter(struct view *view)
 {
 	struct window *window = view->window;
@@ -669,8 +680,7 @@ void window_page_up(struct view *view)
 						     window->columns)))
 				break;
 		display_insert_lines(display, window->row, window->column,
-				     1, window->rows, window->columns,
-				     window->fgrgba, window->bgrgba);
+				     1, window->rows, window->columns);
 	}
 	locus_set(view, CURSOR, start);
 }
@@ -686,8 +696,7 @@ void window_page_down(struct view *view)
 		if (!(bytes = find_row_bytes(view, start, 0, window->columns)))
 			break;
 	display_delete_lines(display, window->row, window->column, 1,
-			     window->rows, window->columns,
-			     window->fgrgba, window->bgrgba);
+			     window->rows, window->columns);
 	new_start(view, start);
 	locus_set(view, CURSOR, start);
 }
@@ -706,8 +715,6 @@ static void window_colors(void)
 	struct window *window, *w;
 
 	static rgba_t colors[][2] = {
-		{ DEFAULT_FGRGBA, DEFAULT_BGRGBA },
-		{ 0x00000000, 0x7f7f7f00 },
 		{ 0x00000000, 0xffffff00 },
 		{ 0x0000ff00, 0xffff0000 },
 		{ 0x0000ff00, 0x7f7f0000 },
@@ -721,12 +728,12 @@ static void window_colors(void)
 	for (window = window_list; window; window = window->next)
 		window->bgrgba = 0;
 	j = active_window != window_list || active_window->next;
-	active_window->fgrgba = colors[j][0];
-	active_window->bgrgba = colors[j][1];
+	active_window->fgrgba = DEFAULT_FGRGBA;
+	active_window->bgrgba = DEFAULT_BGRGBA;
 	for (window = window_list; window; window = window->next) {
 		if (window->bgrgba)
 			continue;
-		for (j = 1; colors[j][1]; j++) {
+		for (j = 0; colors[j][1]; j++) {
 			for (w = window_list; w; w = w->next)
 				if (w != window &&
 				    w->bgrgba == colors[j][1] &&
